@@ -319,7 +319,7 @@ TclMacOSXSetFileAttribute(
     } else {
 	Tcl_WideInt newRsrcForkSize;
 
-	if (Tcl_GetWideIntFromObj(interp, attributePtr,
+	if (TclGetWideIntFromObj(interp, attributePtr,
 		&newRsrcForkSize) != TCL_OK) {
 	    return TCL_ERROR;
 	}
@@ -578,7 +578,7 @@ GetOSTypeFromObj(
     int result = TCL_OK;
 
     if (objPtr->typePtr != &tclOSTypeType) {
-	result = tclOSTypeType.setFromAnyProc(interp, objPtr);
+	result = SetOSTypeFromAny(interp, objPtr);
     }
     *osTypePtr = (OSType) objPtr->internalRep.longValue;
     return result;
@@ -608,7 +608,7 @@ NewOSTypeObj(
     Tcl_Obj *objPtr;
 
     TclNewObj(objPtr);
-    Tcl_InvalidateStringRep(objPtr);
+    TclInvalidateStringRep(objPtr);
     objPtr->internalRep.longValue = (long) osType;
     objPtr->typePtr = &tclOSTypeType;
     return objPtr;
